package cn.fastfun.controller.api;

import cn.fastfun.controller.param.*;
import cn.fastfun.service.AppDeviceLogService;
import cn.fastfun.service.AppDeviceService;
import cn.fastfun.service.entity.AppDevice;
import cn.fastfun.service.entity.AppDeviceLog;
import cn.fastfun.util.DateUtils;
import cn.fastfun.util.ObjectUtil;
import com.bridge.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bridge AutoGen
 */
@Slf4j
@Api(tags = {"2.1 设备操作信息"})
@RestController
@RequestMapping("/api/v1/app/device/log")
public class ApiAppDeviceLogController {
    //业务类
    @Resource(name = "appDeviceService")
    AppDeviceService appDeviceService;

    @Resource
    AppDeviceLogService deviceLogService;

    @ApiOperation(value = "入库")
    @RequestMapping(value = "instorage", method = RequestMethod.POST)
    public ApiDTO inStorage(@RequestBody @ApiParam(name = "入库参数", value = "传入json格式", required = true) LibraryInFormParam param) {
        deviceLogService.putInStorage(param);
        return ApiDTO.ok();
    }

    @ApiOperation(value = "划拨")
    @RequestMapping(value = "transfer", method = RequestMethod.POST)
    public ApiDTO transfer(@RequestBody @ApiParam(name = "划拨参数", value = "传入json格式", required = true) TransferFormParam param) {
        if (CollectionUtils.isEmpty(param.getBatchNum()) && CollectionUtils.isEmpty(param.getSn()))
            return ApiDTO.error("未设定批次号和设备号");
        deviceLogService.transfer(param);
        return ApiDTO.ok();
    }

    @ApiOperation(value = "出库")
    @RequestMapping(value = "outstorage", method = RequestMethod.POST)
    public ApiDTO outStorage(@RequestBody @ApiParam(name = "出库参数", value = "传入json格式", required = true) LibraryOutFormParam param) {
        deviceLogService.outStorage(param);
        return ApiDTO.ok();
    }

    @ApiOperation(value = "出库详细分页搜索")
    @RequestMapping(value = "outstorage/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO outPageQuery(@RequestBody OutQueryParam param) {
        param.addParam(QueryParamExp.eq("type", 3));
        Page<AppDeviceLog> deviceLogPage = deviceLogService.findByParam(param);
        deviceLogPage.getContent().forEach(p -> {
            AppDevice device = new AppDevice();
            device.setSn(p.getSn());
            appDeviceService.addSnTitle(device);
            p.appendFormDevice(device);
        });
        return new ApiPageDTO(null, deviceLogPage);
    }

    @ApiOperation(value = "处置")
    @RequestMapping(value = "handle", method = RequestMethod.POST)
    public ApiDTO handle(@RequestBody @ApiParam(name = "出库参数", value = "传入json格式", required = true) HandelFormParam param) {
        deviceLogService.handle(param);
        return ApiDTO.ok();
    }

    @ApiOperation(value = "处置搜索列表")
    @RequestMapping(value = "pageQuery", method = RequestMethod.POST)
    public ApiPageDTO deviceLogPageQuery(@RequestBody HandelQueryParam param) {
        param.addParam(QueryParamExp.eq("type", 3));
        return new ApiPageDTO(null, deviceLogService.findByParam(param));
    }

    @ApiOperation(value = "调拨详细搜索")
    @RequestMapping(value = "transfer/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO pageQuery(@RequestBody BatchNumQueryParam param) {
        Page<AppDeviceLog> deviceLogPage = deviceLogService.findByParam(param);

        deviceLogPage.getContent().forEach(p -> {
            AppDevice device = new AppDevice();
            device.setSn(p.getSn());
            appDeviceService.addSnTitle(device);
            p.appendFormDevice(device);
        });
        return new ApiPageDTO(null, deviceLogPage);
    }


    @ApiOperation(value = "调拨分页搜索列表")
    @RequestMapping(value = "batchNum/transfer/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO batchNumPageQuery(@RequestBody LogTransferQueryParam param) {
        StringBuffer sql = new StringBuffer("select t.batch_num,t.add_time,count(0) as total,t.tf_event,t.tf_device_type,t.tf_used,t.tf_describe,t.out_custom_id,t.operator,t.remarks from app_device_log t where t.type = 2");

        // sn搜索
        if (!StringUtils.isEmpty(param.getSn())) {
            sql.append(" where t.sn like '%").append(param.getSn()).append("%'");
        }

        // 类型搜索
        if (!StringUtils.isEmpty(param.getType())) {
            sql.append(StringUtils.isEmpty(param.getSn()) ? "where" : "and");
            sql.append(" t.tf_device_type = '").append(param.getType()).append("'");
        }
        sql.append(" group by t.batch_num,t.add_time,t.tf_event,t.tf_device_type,t.tf_used,t.tf_describe,t.out_custom_id,t.operator,t.remarks");
        if (!StringUtils.isEmpty(param.getOrderBy())) {
            sql.append(param.getOrderBy());
        }
        log.info("SQL: {}", sql.toString());

        ApiPageDTO page = appDeviceService.getListBySQL(sql.toString(), new HashMap<>(), param);
        // 组装数据
        if(null !=  page.getData()) {
            List<Map<String, Object>> list = (List<Map<String, Object>>) page.getData();
            for (Map<String, Object> p : list) {
                p.put("add_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("add_time")), DateUtils.YMDHMS));
                p.put("customTitle", "骑享科技");
                p.put("operator", "admin");
            }
            page.setData(list);
        }
        return page;
    }


    @ApiOperation(value = "出库分页搜索列表")
    @RequestMapping(value = "batchNum/out/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO outPageQuery(@RequestBody LogTransferQueryParam param) {
        StringBuffer sql = new StringBuffer("select t.batch_num,t.add_time,count(0) as total,t.out_type,t.receiver_name,t.receiver_phone,t.remarks,t.out_custom_id,t.operator,t.update_time from app_device_log t where t.type = 3");

        // sn搜索
        if (!StringUtils.isEmpty(param.getSn())) {
            sql.append(" where t.sn like '%").append(param.getSn()).append("%'");
        }

        // 类型搜索
        if (!StringUtils.isEmpty(param.getType())) {
            sql.append(StringUtils.isEmpty(param.getSn()) ? "where" : "and");
            sql.append(" t.out_type = '").append(param.getType()).append("'");
        }
        sql.append(" group by t.batch_num,t.add_time,t.out_type,t.receiver_name,t.receiver_phone,t.remarks,t.out_custom_id,t.operator,t.update_time");
        if (!StringUtils.isEmpty(param.getOrderBy())) {
            sql.append(param.getOrderBy());
        }
        log.info("SQL: {}", sql.toString());

        ApiPageDTO page = appDeviceService.getListBySQL(sql.toString(), new HashMap<>(), param);
        if (null != page.getData()) {
            // 组装数据
            List<Map<String, Object>> list = (List<Map<String, Object>>) page.getData();
            for (Map<String, Object> p : list) {
                p.put("add_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("add_time")), DateUtils.YMDHMS));
                p.put("update_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("update_time")), DateUtils.YMDHMS));
                p.put("operator", "admin");
                p.put("out_type",String.valueOf(p.get("out_type")));
            }
            page.setData(list);
        }
        return page;
    }

}